﻿##### Scriptinfo ##################################################################################
# Scriptreihe	    PAM-AdminGUI\ProxyFunctions
# Datum:            2017-10-16
# Version:          V1.03
# Programmierer:    Stephan Walther
###################################################################################################

# lese AD-Infos und Variablen    
    Get-Content -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI.ini" | 
        ForEach-Object {
            if ($_.length -gt 0) {
                if ('[' -notcontains ($_)[0]) {
                    $Name,$Wert = ($_ -split '#')[0].Trim() -split '=='

                    $Name = "WS_" + $Name.trim()

                    Get-Variable -Scope global | Where-Object {$_.name -eq $Name} | Remove-Variable -Scope global
                    New-Variable -Name $Name.trim() -Value $Wert.trim() -Scope global
                }
            }
        }

    $Gruppen = @()
    if ($WS_MitDomainAdmins     -eq 'an') { $Gruppen += (Get-ADGroup -Filter 'Admincount -eq 1' | Where-Object {$_.SID -like '*-512'}).SamAccountName } 
    if ($WS_MitSchemaAdmins     -eq 'an') { $Gruppen += (Get-ADGroup -Filter 'Admincount -eq 1' | Where-Object {$_.SID -like '*-518'}).SamAccountName } 
    if ($WS_MitEnterpriseAdmins -eq 'an') { $Gruppen += (Get-ADGroup -Filter 'Admincount -eq 1' | Where-Object {$_.SID -like '*-519'}).SamAccountName }         
    $Gruppen += (Get-ADGroup -Filter ($WS_FilterAdminGruppen -replace '"') -SearchBase ($WS_OUAdminGruppen -replace '"')).SamAccountName    
    $Gruppen    = "'" + ($Gruppen -join "','") + "'"
    
    $Benutzer   = "'" + ((Get-ADUser -Filter ($WS_FilterAdmins -replace '"') -SearchBase ($WS_OUAdmins -replace '"')).SamAccountName -join "','") + "'"
    $Zeitspanne = $WS_AdminZeitInMinuten
    
# Function erstelle-PAMGruppenMitglied
$Function = @'
function erstelle-PAMGruppenMitglied {
    [cmdletbinding()]
    param(
        [parameter(Mandatory=$true)] [ValidateSet( %GRUPPEN%  )]   [string] $Gruppe,
        [parameter(Mandatory=$true)] [ValidateSet( %BENUTZER% )]   [string] $Admin,
        [parameter(Mandatory=$true)] [ValidateSet( %ZEITSPANNE% )] [string] $Zeitspanne
    )

    try {
        $Error.Clear()
        Add-ADGroupMember -Identity $Gruppe -Members $Admin -MemberTimeToLive (New-TimeSpan -Minutes $Zeitspanne) -ErrorAction SilentlyContinue
        Return 'OK'
    } 
    catch {
        Return ($error[0].Exception.Message)
    }
}
'@

$Function = $Function -replace '%GRUPPEN%',$Gruppen
$Function = $Function -replace '%BENUTZER%',$Benutzer
$Function = $Function -replace '%ZEITSPANNE%',$Zeitspanne
Invoke-Expression -Command $Function

# Function entferne-PAMGruppenMitglied
$Function = @'
function entferne-PAMGruppenMitglied {
    [cmdletbinding()]
    param(
        [parameter(Mandatory=$true)] [ValidateSet( %GRUPPEN%  )] [string] $Gruppe,
        [parameter(Mandatory=$true)] [ValidateSet( %BENUTZER% )] [string] $Admin
    )

    try {
        $Error.Clear()

        $Member = (Get-ADGroup -Identity $Gruppe -ShowMemberTimeToLive -Properties members).members |
            Where-Object {$_ -like "<TTL=*,CN=$Admin,*"}

        $Member = $Member -split '>,'
        if ($Member.Count -eq 2) {
            Remove-ADGroupMember -Identity $Gruppe -Members $Member[1] -Confirm:$false
            Return "OK"
        } else {
            Return "Statische Gruppenmitglieder können nicht entfernt werden!" 
        }
    } 
    catch {
        Return ($error[0].Exception.Message)
    }
}
'@

$Function = $Function -replace '%GRUPPEN%',$Gruppen
$Function = $Function -replace '%BENUTZER%',$Benutzer
Invoke-Expression -Command $Function

# Function liste-Admins
$Function = @"
function liste-Admins {
    Return ($Benutzer -split ',' | Sort-Object)
}
"@
Invoke-Expression -Command $Function

# Function liste-Gruppen
$Function = @"
function liste-Gruppen {
    Return ($Gruppen -split ',' | Sort-Object)
}
"@
Invoke-Expression -Command $Function

# Function liste-Zeitspannen
$Function = @"
function liste-Zeitspannen {
    Return ([int[]] ($Zeitspanne -split ',') | Sort-Object)
}
"@
Invoke-Expression -Command $Function

# Function zeige-GruppenMitglieder
$Function = @'
function zeige-GruppenMitglieder {
    param (
        [parameter(Mandatory=$true)] [ValidateSet( %GRUPPEN%  )] [string] $Gruppe
    )
    (Get-ADGroup -Identity $Gruppe -ShowMemberTimeToLive -Properties members).members | 
        ForEach-Object {   
            $User = $_ -split '>,'
            if ($User.count -gt 1) {$User = $User[1]} else {$User = $User[0]}
            
            if ((Get-ADObject $User).ObjectClass -eq 'user') { 
                #if ($User.Count -gt 1) {$USer = $User[1]} else {$USer = $User[0]}           

                $SAM = (Get-ADUser -Identity $User).SamAccountName
                $TTL  = (($_ -split 'CN=')[0] -split '=')[-1] -replace '>' -replace ','
                $bis  = (($_ -split 'CN=')[0] -split '=')[-1] -replace '>'
                if ($bis) {
                    $bis = get-date -date (Get-Date).AddSeconds($bis) -Format T
                }

                $out = New-Object -TypeName psobject
                $out | Add-Member -MemberType NoteProperty -Name 'User' -Value $SAM
                $out | Add-Member -MemberType NoteProperty -Name 'TTL'  -Value $TTL
                $out | Add-Member -MemberType NoteProperty -Name 'bis'  -Value $bis
                $out
            } 
        }
}
'@
$Function = $Function -replace '%GRUPPEN%',$Gruppen
Invoke-Expression -Command $Function

# Function zeige-Mitgliedschaften
$Function = @'
function zeige-Mitgliedschaften {
    param (
        [parameter(Mandatory=$true)] [ValidateSet( %BENUTZER% )] [string] $Admin
    )
        
    $AdminGruppen = ($Gruppen -replace "'") -split ','

    (Get-ADUser -Identity $Admin -Properties memberof).memberof | 
        ForEach-Object {
            $Gruppe = (Get-ADGroup -Identity $_).SamAccountName        
            $Member = (Get-ADGroup -Identity $Gruppe -ShowMemberTimeToLive -Properties members).members 
            
            if ($AdminGruppen -contains $Gruppe) {
                $Gruppe = (Get-ADGroup -Identity $Gruppe).SamAccountName
                $TTL    = (($Member -split 'CN=')[0] -split '=')[-1] -replace '>'            
                if ($TTL) {
                    $bis = get-date -date (Get-Date).AddSeconds($TTL) -Format T
                } else {
                    $bis = ''
                }

                $out = New-Object -TypeName psobject
                $out | Add-Member -MemberType NoteProperty -Name 'Gruppe' -Value $Gruppe
                $out | Add-Member -MemberType NoteProperty -Name 'TTL'    -Value $TTL
                $out | Add-Member -MemberType NoteProperty -Name 'bis'    -Value $bis
                $out
            }        
        }
}
'@
$Function = $Function -replace '%BENUTZER%',$Benutzer
Invoke-Expression -Command $Function

# SIG # Begin signature block
# MIII4QYJKoZIhvcNAQcCoIII0jCCCM4CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUx94RcxwrIOtkXr+EYuGeWGDd
# BL6gggY/MIIGOzCCBSOgAwIBAgITGwAAAOBAvnzebcAjSAABAAAA4DANBgkqhkiG
# 9w0BAQsFADBUMRMwEQYKCZImiZPyLGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYC
# d3MxKTAnBgNVBAMTIFdTLUlUUy1aZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExMB4X
# DTE3MDUxMDA1NDI0OVoXDTE5MDUxMDA1NTI0OVowdTETMBEGCgmSJomT8ixkARkW
# A2l0czESMBAGCgmSJomT8ixkARkWAndzMQswCQYDVQQLEwJXUzERMA8GA1UECxMI
# QmVudXR6ZXIxDzANBgNVBAsTBldTLUlUUzEZMBcGA1UEAxMQV2FsdGhlciwgU3Rl
# cGhhbjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKqnKBq0nJLvqGwO
# aJkLU2174/KhLd4ykJZ2BgG4wse7s3ALM7dH1vxNmeKIoB09yk9nYZzDeRjZ7w7a
# WKgko8zvNHJCleQNtG6AReOoMIRuYyrbUmbKlmICn9V/ENUNQ6s1tiqnAIChb0J2
# 05GMaBYG+IFS+uLDzlTIsoaEKZjV3htgBEVQt6BYIdbsZ913ZUcpGWhQsCXKairT
# wB8pqPTaBCA5HJHyfAwxa+w2ERhF5q0r/xp9dEkemFjy0Xo9aQWUw3WPy8l3pTyp
# I2w/yDFqLfZu5z1QjWrcNA9VirSiM9PkOPXQb9riChc4PfFd90+JjeVhOlj3y0qb
# InQsfUMCAwEAAaOCAuMwggLfMA4GA1UdDwEB/wQEAwIHgDA8BgkrBgEEAYI3FQcE
# LzAtBiUrBgEEAYI3FQiIoH6B98FWhLmPAobjm2SE8KUnMYPz1QqE8ow3AgFkAgEC
# MB0GA1UdDgQWBBT1lyML2Qw04hBbAlX5PeqiA5IBQjAfBgNVHSMEGDAWgBS1Nrmv
# a6rveo5pQqKi2+u+kQ59cTCCASAGA1UdHwSCARcwggETMIIBD6CCAQugggEHhoHE
# bGRhcDovLy9DTj1XUy1JVFMtWmVydGlmaXppZXJ1bmdzc3RlbGxlLUNBMSgxKSxD
# Tj1XUy1DQTEsQ049Q0RQLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNl
# cnZpY2VzLENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NlcnRpZmljYXRl
# UmV2b2NhdGlvbkxpc3Q/YmFzZT9vYmplY3RDbGFzcz1jUkxEaXN0cmlidXRpb25Q
# b2ludIY+aHR0cDovL2NybC53cy5pdHMvY3JsZC9XUy1JVFMtWmVydGlmaXppZXJ1
# bmdzc3RlbGxlLUNBMSgxKS5jcmwwgc0GCCsGAQUFBwEBBIHAMIG9MIG6BggrBgEF
# BQcwAoaBrWxkYXA6Ly8vQ049V1MtSVRTLVplcnRpZml6aWVydW5nc3N0ZWxsZS1D
# QTEsQ049QUlBLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNlcnZpY2Vz
# LENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NBQ2VydGlmaWNhdGU/YmFz
# ZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9uQXV0aG9yaXR5MBMGA1UdJQQMMAoG
# CCsGAQUFBwMDMBsGCSsGAQQBgjcVCgQOMAwwCgYIKwYBBQUHAwMwKQYDVR0RBCIw
# IKAeBgorBgEEAYI3FAIDoBAMDnN0ZXBoYW5Ad3MuaXRzMA0GCSqGSIb3DQEBCwUA
# A4IBAQBjyTnnRXeyMEPW8SziBVFJcsX0K3XLCXz7PEl6sVuiWTHY3n9IOwJJ2sqF
# 3Sea0bpWrDFnJQtB+S4FXW6A/bVvuPlGMBtZ7T9DowowAQAmWvC/OJeeHGz2xyjh
# fDYAmMi3DXrVIEv+yWHsrh6CSt0KiHjMOh/Kk092AmyyPSRZpqmabzcoY7fI6xff
# ckOvS+Q3233ytXfS2YMuNKL8EuGaUhGj8Smb3grUtNW0kyJmL2wTgyWdUerbGpC3
# Ss7iv53hJXSoe8Rmo8spntb/5e+zt6q0ThqI4TN6sm9Gqgg8QcfzGc0XvW7V2GKU
# LT75dOMzV86lFZo+rvq1SaZb2GRcMYICDDCCAggCAQEwazBUMRMwEQYKCZImiZPy
# LGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYCd3MxKTAnBgNVBAMTIFdTLUlUUy1a
# ZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExAhMbAAAA4EC+fN5twCNIAAEAAADgMAkG
# BSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJ
# AzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMG
# CSqGSIb3DQEJBDEWBBRYqklfzhqpYP0mvGus4c9SeO6NGzANBgkqhkiG9w0BAQEF
# AASCAQARZO/Vo/0qlprJtQAtjWBEt+RzfvbZwXd9/myv0YA8qRRogSki5cVRYRGu
# ySVlFv5ApPu+rBJpDrgfWA7OLbgVu2q84uIIw/YPl2btlGMyAV4Tu2ysRHe0UTYE
# H5GqwXIyHmGRMy4Uz8VI6tMz25XxC+ozCk48OXGrhkpUtYN0BduiYa6pPgOy1aPo
# nPgYJDQE9xYBeZJk8ImiCsmvvMcK9QCiE3DKUx9AC4eqFMbFbjyZDvF9RxYMGqS8
# df3VT41Bt26kHCoLR3APK4MF2Y7SAZXhtuEs2yJsT1QNzQl6XJZ8IkjHQikt1ndl
# k5e4kWMyTQrL5KFSabp7TZ+2XHsi
# SIG # End signature block
