﻿##### Scriptinfo ##################################################################################
# Scriptreihe	    PAM-AdminGUI\UserGUI
# Datum:            2017-11-15
# Version:          V1.04
# Programmierer:    Stephan Walther
###################################################################################################

[cmdletbinding()]
param(    
    $PAMServer = 'C-DC1.CRASHWORK.GLOBAL',    
    $PAMRolle  = 'PAM-AdminGUI'    
)

# Variablen, Typen und Bibliotheken ###############################################################

cls

[void] [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms") 
[void] [System.Reflection.Assembly]::LoadWithPartialName("System.Drawing") 
[System.Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic') | Out-Null
[System.Windows.Forms.Application]::EnableVisualStyles()

Add-Type -Name Window -Namespace Console -MemberDefinition '
[DllImport("Kernel32.dll")]
public static extern IntPtr GetConsoleWindow();
 
[DllImport("user32.dll")]
public static extern bool ShowWindow(IntPtr hWnd, Int32 nCmdShow);
'

$globaleVariablen = 'Admins','Gruppen','Modus','JEASession'

$globaleVariablen | ForEach-Object {
    $VarName = $_
    Get-Variable -Scope global |Where-Object {$_.name -eq $VarName} | Remove-Variable -Scope global
    New-Variable -Name $VarName -Value @() -Scope global    
}

$ScriptVersion = ((Get-Content -Path 'PAM-AdminGUI.ps1' | Select-String '# Version:' | Where-Object { $_ -notlike '*get-content*' }) -split ':')[1].trim()

# Funktionsdeklarationen ##########################################################################

function Main {
    $PSScriptRoot = $MyInvocation.PSScriptRoot
    Set-Location $PSScriptRoot
    
    Hide-Console | Out-Null              
             
    $error.Clear()
    $global:JEASession = New-PSSession -ComputerName $PAMServer -ConfigurationName $PAMRolle -ErrorAction SilentlyContinue
    if ($error) {
        [System.Windows.Forms.MessageBox]::Show("Es konnte keine Verbindung zum JEA-Endpunkt $PAMServer\$PAMRolle aufgebaut werden!" , "FEHLER") | Out-Null
    } else { 
        erzeuge-Hauptformular 
    }

    Get-PSSession | Remove-PSSession
    Show-Console | Out-Null
}

function ermittle-Admins {    
    $global:admins = Invoke-Command -Session $JEASession -ScriptBlock {
        liste-Admins
    } | Sort-Object   
}

function ermittle-Gruppen {    
    $global:Gruppen = Invoke-Command -Session $JEASession -ScriptBlock {
        liste-Gruppen
    } | Sort-Object   
}

function ermittle-Zeitspannen {
    $Zeitspannen = Invoke-Command -Session $JEASession -ScriptBlock {
        liste-Zeitspannen
    } 

    $Zeitspannen | ForEach-Object { $obj_Zeitraum.items.add($_) | Out-Null }
    if ($Zeitspannen.count -gt 0) { $obj_Zeitraum.Text = $Zeitspannen[0] }
}

function aktualisiere-SelektorListe {
    $form_Main.cursor = [System.Windows.Forms.Cursors]::WaitCursor
    $lbl_Footer.Text = 'lade Daten'
    
    if ($Modus -eq 'Gruppen') {        
        $Liste = $Gruppen
    } else {
        $Liste = $Admins
    }

    $obj_Liste_Selector.Items.clear()
    $Liste | ForEach-Object { $obj_Liste_Selector.Items.Add($_) | Out-Null }
    
    $form_Main.cursor = [System.Windows.Forms.Cursors]::Default        
    $lbl_Footer.Text = 'bereit'
}

function aktualisiere-ListeYes {
    $Selector = $obj_Liste_Selector.SelectedItem  
    
    $form_Main.cursor = [System.Windows.Forms.Cursors]::WaitCursor
    $lbl_Footer.Text = 'lade Daten'
    
    if ($Modus -eq 'Gruppen') {
        $Liste = Invoke-Command -Session $JEASession -ScriptBlock {
            param($Gruppe) 
            zeige-GruppenMitglieder -Gruppe $Gruppe 
        } -ArgumentList $Selector 
        
        $obj_Liste_Yes.items.Clear()
        $Liste | Sort-Object | ForEach-Object { 
            $User = $_.user
            $TTL  = $_.TTL
            $bis  = $_.bis
            if ($TTL) {
                $obj_Liste_Yes.Items.Add($User + " (noch bis $bis)") | Out-Null 
            } else {
                $obj_Liste_Yes.Items.Add($User) | Out-Null 
            }
        }
    } else {
        $Liste = Invoke-Command -Session $JEASession -ScriptBlock {
            param($Admin) 
            zeige-Mitgliedschaften -Admin $Admin 
        } -ArgumentList $Selector 
        
        $obj_Liste_Yes.items.Clear()
        $Liste | Sort-Object -Property Gruppe | ForEach-Object { 
            $Gruppe = $_.Gruppe
            $TTL    = $_.TTL
            $bis    = $_.bis
            if ($TTL) {
                $obj_Liste_Yes.Items.Add($Gruppe + " (noch bis $bis)") | Out-Null 
            } else {
                $obj_Liste_Yes.Items.Add($Gruppe) | Out-Null 
            }
        }
    }

    $form_Main.cursor = [System.Windows.Forms.Cursors]::Default        
    $lbl_Footer.Text = 'bereit'
}

function aktualisiere-ListeNo {
    $ListeYes = @()
    $obj_Liste_Yes.Items | ForEach-Object {
        $ListeYes += ($_ -split ' \(')[0]
    }

    if ($Modus -eq 'Gruppen') { $Liste = $admins } else { $Liste = $Gruppen}
    
    $obj_Liste_no.items.Clear()    
    $Liste | ForEach-Object {
        if ($ListeYes -notcontains $_) {
            $obj_Liste_No.Items.Add($_) #| Out-Null 
        } 
    }
}

function erzeuge-Hauptformular {    
    $form_Main = New-Object System.Windows.Forms.Form 
    $form_Main.cursor = [System.Windows.Forms.Cursors]::WaitCursor
    $form_Main.Text = "$PAMRolle - verbunden mit $PAMServer (Version $ScriptVersion)"
    $form_Main.Size = '800,495'
    $form_Main.StartPosition = "CenterScreen"
    $form_Main.Topmost = $false
    $form_Main.KeyPreview = $True
    $Logo = (Get-Item -Path .\wsits.ico -ErrorAction SilentlyContinue).FullName
    if ($Logo) { $form_Main.Icon = $Logo }

    $lbl_Modus = New-Object System.Windows.Forms.Label
    $lbl_Modus.Location = '10,10'
    $lbl_Modus.Size = '150,20'
    $lbl_Modus.Text = "Modus:"
    $form_Main.Controls.Add($lbl_Modus) 

    $obj_Modus = New-Object System.Windows.Forms.ComboBox
    $obj_Modus.Location = '160,10'
    $obj_Modus.Size = '200,20'
    $obj_Modus.Text = "Gruppen"
    $obj_Modus.items.add('Gruppen') | Out-Null
    $obj_Modus.items.add('Admins') | Out-Null
    $obj_Modus.Enabled = $true    
    $obj_Modus.add_SelectedIndexChanged({
        $global:Modus = $obj_Modus.SelectedItem
                
        if ($Modus -eq 'Admins') {            
            $lbl_Selector.Text = "Admins:"
            $lbl_ListeNo.Text  = "mögliche Gruppen:"
            $lbl_ListeYes.Text = "Mitglied:"
        } else {            
            $lbl_Selector.Text = "Gruppen:"
            $lbl_ListeNo.Text  = "mögliche Admins:"
            $lbl_ListeYes.Text = "Mitglieder:"
        }
        aktualisiere-SelektorListe        
        aktualisiere-ListeNo
        $obj_Liste_Yes.items.clear()
    })
    $form_Main.Controls.Add($obj_Modus)

    $lbl_Zeitraum = New-Object System.Windows.Forms.Label
    $lbl_Zeitraum.Location = '10,35'
    $lbl_Zeitraum.Size = '150,20'
    $lbl_Zeitraum.Text = "Zeitraum [min]:"
    $form_Main.Controls.Add($lbl_Zeitraum) 

    $obj_Zeitraum = New-Object System.Windows.Forms.ComboBox
    $obj_Zeitraum.Location = '160,35'
    $obj_Zeitraum.Size = '200,20'
    $obj_Zeitraum.Text = ""    
    $obj_Zeitraum.Enabled = $true    
    $form_Main.Controls.Add($obj_Zeitraum)

    $Button_CMD = New-Object System.Windows.Forms.Button
    $Button_CMD.Location = '680,10'
    $Button_CMD.Size = '100,25'
    $Button_CMD.Text = "zeige CMD" 
    $Button_CMD.Enabled = $true                                                                                                                     
    $Button_CMD.Add_Click({ 
        if ($Button_CMD.text -eq 'zeige CMD') {
            Show-Console | Out-Null
            $Button_CMD.text = 'verstecke CMD' 
        } else {
            Hide-Console | Out-Null    
            $Button_CMD.text = 'zeige CMD' 
        }    
    })
    $form_Main.Controls.Add($Button_CMD) 
    

        
    $lbl_Selector = New-Object System.Windows.Forms.Label
    $lbl_Selector.Location = '10,60'
    $lbl_Selector.Size = '250,20'
    $lbl_Selector.Text = "Gruppen:"
    $form_Main.Controls.Add($lbl_Selector) 
    
    $obj_Liste_Selector = New-Object System.Windows.Forms.ListBox
    $obj_Liste_Selector.Location = '10,85'
    $obj_Liste_Selector.Size = '250,300'
    $obj_Liste_Selector.add_SelectedIndexChanged({ 
        aktualisiere-ListeYes
        aktualisiere-ListeNo

        $Button_del.Enabled = $false 
        $Button_add.Enabled = $false
    })
    $form_Main.Controls.Add($obj_Liste_Selector) 



    $lbl_ListeNo = New-Object System.Windows.Forms.Label
    $lbl_ListeNo.Location = '270,60'
    $lbl_ListeNo.Size = '250,20'
    $lbl_ListeNo.Text = "mögliche Admins:"
    $form_Main.Controls.Add($lbl_ListeNo) 
        
    $obj_Liste_No = New-Object System.Windows.Forms.ListBox
    $obj_Liste_No.Location = '270,85'
    $obj_Liste_No.Size = '250,300'
    $obj_Liste_No.add_SelectedIndexChanged({ 
        $Button_add.Enabled = $true      
    })
    $form_Main.Controls.Add($obj_Liste_No) 
         
    $Button_add = New-Object System.Windows.Forms.Button
    $Button_add.Location = '270,380'
    $Button_add.Size = '100,25'
    $Button_add.Text = "hinzufügen" 
    $Button_add.Enabled = $false                                                                                                                     
    $Button_add.Add_Click({    
        if ($Modus -eq 'Gruppen') {
            $Gruppe = $obj_Liste_Selector.SelectedItem
            $Admin  = $obj_Liste_No.SelectedItem
        } else {
            $Gruppe = $obj_Liste_No.SelectedItem
            $Admin  = $obj_Liste_Selector.SelectedItem
        }        
        
        $Zeit   = $obj_Zeitraum.SelectedItem
        
        $Ergebnis = Invoke-Command -Session $JEASession -ScriptBlock {
            param($Gruppe,$Admin,$Zeit)

            erstelle-PAMGruppenMitglied -Admin $Admin -Gruppe $Gruppe -Zeitspanne $Zeit 
        } -ArgumentList $Gruppe,$Admin,$Zeit
        
        if ($Ergebnis -eq 'OK') {            
            aktualisiere-ListeYes
            aktualisiere-ListeNo

            if ($Modus -eq 'Gruppen') {
                $CurItem = $obj_Liste_Yes.items | Where-Object {$_ -like "$Admin*"}
            } else {
                $CurItem = $obj_Liste_Yes.items | Where-Object {$_ -like "$Gruppe*"}
            }            
            $obj_Liste_Yes.SetSelected($obj_Liste_Yes.FindString($CurItem),$true) 

            $Button_del.Enabled = $true      
            $Button_add.Enabled = $false      
        } else {
            [System.Windows.Forms.MessageBox]::Show("$Ergebnis" , "FEHLER")   
        }
    })    
    $form_Main.Controls.Add($Button_add)
    


    $lbl_ListeYes = New-Object System.Windows.Forms.Label
    $lbl_ListeYes.Location = '530,60'
    $lbl_ListeYes.Size = '250,20'
    $lbl_ListeYes.Text = "Mitglieder:"
    $form_Main.Controls.Add($lbl_ListeYes) 
        
    $obj_Liste_Yes = New-Object System.Windows.Forms.ListBox
    $obj_Liste_Yes.Location = '530,85'
    $obj_Liste_Yes.Size = '250,300'
    $obj_Liste_Yes.add_SelectedIndexChanged({ 
        $Button_del.Enabled = $true      
    })
    $form_Main.Controls.Add($obj_Liste_Yes) 
         
    $Button_del = New-Object System.Windows.Forms.Button
    $Button_del.Location = '530,380'
    $Button_del.Size = '100,25'
    $Button_del.Text = "entfernen" 
    $Button_del.Enabled = $false                                                                                                                     
    $Button_del.Add_Click({    
        if ($Modus -eq 'Gruppen') {
            $Gruppe = $obj_Liste_Selector.SelectedItem
            $Admin  = ($obj_Liste_Yes.SelectedItem -split ' \(')[0]
        } else {
            $Gruppe = ($obj_Liste_Yes.SelectedItem -split ' \(')[0]
            $Admin  = $obj_Liste_Selector.SelectedItem
        }
        
        $Ergebnis = Invoke-Command -Session $JEASession -ScriptBlock {
            param($Gruppe,$Admin)

            entferne-PAMGruppenMitglied -Admin "$Admin" -Gruppe "$Gruppe"
        } -ArgumentList $Gruppe,$Admin

        if ($Ergebnis -eq 'OK') {            
            aktualisiere-ListeYes
            aktualisiere-ListeNo

            $Button_del.Enabled = $false      
            $Button_add.Enabled = $false      
        } else {
            if ($Ergebnis -eq $null) {$Ergebnis = $error[0].exception.message}

            [System.Windows.Forms.MessageBox]::Show("$Ergebnis" , "FEHLER")   
        }
    })    
    $form_Main.Controls.Add($Button_del)
    


    $lbl_Footer = New-Object 'System.Windows.Forms.Label'
    $lbl_Footer.Location = '10,440'
    $lbl_Footer.Size = "$($form_Main.Height - 30),20"
    $lbl_Footer.text = 'Bereit'    
    $form_Main.Controls.Add($lbl_Footer)
           
    $form_Main.ResumeLayout()	
    $InitialFormWindowState = $form_Main.WindowState
    $form_Main.Add_Shown({
        $form_Main.Activate()

        $global:Modus = 'Gruppen'

        ermittle-Zeitspannen
        ermittle-Admins   
        ermittle-Gruppen
        aktualisiere-SelektorListe
    })    	
	$form_Main.add_Load({
        
    })
    [void] $form_Main.ShowDialog()    
}

function Show-Console {
    $consolePtr = [Console.Window]::GetConsoleWindow()
    [Console.Window]::ShowWindow($consolePtr, 5)
}
 
function Hide-Console {
    $consolePtr = [Console.Window]::GetConsoleWindow()
    [Console.Window]::ShowWindow($consolePtr, 0)
}

# Programmstart ###################################################################################

MAIN

# SIG # Begin signature block
# MIII4QYJKoZIhvcNAQcCoIII0jCCCM4CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUvLRcsmmgiN3t2mtxcs8NwHtx
# N+mgggY/MIIGOzCCBSOgAwIBAgITGwAAAOBAvnzebcAjSAABAAAA4DANBgkqhkiG
# 9w0BAQsFADBUMRMwEQYKCZImiZPyLGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYC
# d3MxKTAnBgNVBAMTIFdTLUlUUy1aZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExMB4X
# DTE3MDUxMDA1NDI0OVoXDTE5MDUxMDA1NTI0OVowdTETMBEGCgmSJomT8ixkARkW
# A2l0czESMBAGCgmSJomT8ixkARkWAndzMQswCQYDVQQLEwJXUzERMA8GA1UECxMI
# QmVudXR6ZXIxDzANBgNVBAsTBldTLUlUUzEZMBcGA1UEAxMQV2FsdGhlciwgU3Rl
# cGhhbjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKqnKBq0nJLvqGwO
# aJkLU2174/KhLd4ykJZ2BgG4wse7s3ALM7dH1vxNmeKIoB09yk9nYZzDeRjZ7w7a
# WKgko8zvNHJCleQNtG6AReOoMIRuYyrbUmbKlmICn9V/ENUNQ6s1tiqnAIChb0J2
# 05GMaBYG+IFS+uLDzlTIsoaEKZjV3htgBEVQt6BYIdbsZ913ZUcpGWhQsCXKairT
# wB8pqPTaBCA5HJHyfAwxa+w2ERhF5q0r/xp9dEkemFjy0Xo9aQWUw3WPy8l3pTyp
# I2w/yDFqLfZu5z1QjWrcNA9VirSiM9PkOPXQb9riChc4PfFd90+JjeVhOlj3y0qb
# InQsfUMCAwEAAaOCAuMwggLfMA4GA1UdDwEB/wQEAwIHgDA8BgkrBgEEAYI3FQcE
# LzAtBiUrBgEEAYI3FQiIoH6B98FWhLmPAobjm2SE8KUnMYPz1QqE8ow3AgFkAgEC
# MB0GA1UdDgQWBBT1lyML2Qw04hBbAlX5PeqiA5IBQjAfBgNVHSMEGDAWgBS1Nrmv
# a6rveo5pQqKi2+u+kQ59cTCCASAGA1UdHwSCARcwggETMIIBD6CCAQugggEHhoHE
# bGRhcDovLy9DTj1XUy1JVFMtWmVydGlmaXppZXJ1bmdzc3RlbGxlLUNBMSgxKSxD
# Tj1XUy1DQTEsQ049Q0RQLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNl
# cnZpY2VzLENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NlcnRpZmljYXRl
# UmV2b2NhdGlvbkxpc3Q/YmFzZT9vYmplY3RDbGFzcz1jUkxEaXN0cmlidXRpb25Q
# b2ludIY+aHR0cDovL2NybC53cy5pdHMvY3JsZC9XUy1JVFMtWmVydGlmaXppZXJ1
# bmdzc3RlbGxlLUNBMSgxKS5jcmwwgc0GCCsGAQUFBwEBBIHAMIG9MIG6BggrBgEF
# BQcwAoaBrWxkYXA6Ly8vQ049V1MtSVRTLVplcnRpZml6aWVydW5nc3N0ZWxsZS1D
# QTEsQ049QUlBLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNlcnZpY2Vz
# LENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NBQ2VydGlmaWNhdGU/YmFz
# ZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9uQXV0aG9yaXR5MBMGA1UdJQQMMAoG
# CCsGAQUFBwMDMBsGCSsGAQQBgjcVCgQOMAwwCgYIKwYBBQUHAwMwKQYDVR0RBCIw
# IKAeBgorBgEEAYI3FAIDoBAMDnN0ZXBoYW5Ad3MuaXRzMA0GCSqGSIb3DQEBCwUA
# A4IBAQBjyTnnRXeyMEPW8SziBVFJcsX0K3XLCXz7PEl6sVuiWTHY3n9IOwJJ2sqF
# 3Sea0bpWrDFnJQtB+S4FXW6A/bVvuPlGMBtZ7T9DowowAQAmWvC/OJeeHGz2xyjh
# fDYAmMi3DXrVIEv+yWHsrh6CSt0KiHjMOh/Kk092AmyyPSRZpqmabzcoY7fI6xff
# ckOvS+Q3233ytXfS2YMuNKL8EuGaUhGj8Smb3grUtNW0kyJmL2wTgyWdUerbGpC3
# Ss7iv53hJXSoe8Rmo8spntb/5e+zt6q0ThqI4TN6sm9Gqgg8QcfzGc0XvW7V2GKU
# LT75dOMzV86lFZo+rvq1SaZb2GRcMYICDDCCAggCAQEwazBUMRMwEQYKCZImiZPy
# LGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYCd3MxKTAnBgNVBAMTIFdTLUlUUy1a
# ZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExAhMbAAAA4EC+fN5twCNIAAEAAADgMAkG
# BSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJ
# AzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMG
# CSqGSIb3DQEJBDEWBBSWx6LQJxkDL4clDURg6+Oc6JcUbjANBgkqhkiG9w0BAQEF
# AASCAQArMhTlo019IsH8jmm1guC0LPcihSfJ6acAC1HOhHoAy5cV1XzmST+g4zSc
# UPEZnqac8WJP6IJ/ew25LH9aBVCFBcyou9aL2rWJJzaD/n2Yk2/dFjKc77yYrEsG
# q9p6ahq8woBSK2IA3HtaTGckQz75ncHGWiOHRX04Ptr1+GZVQc+S6Nv3W3GAhXXQ
# 0u9z6m7VDaGLzn3TLdkg1o2EYTs+i4kRqLrdeoHRyMWOAPPqjQwXY/lBCJAKQ62Z
# MXt4/GL70ejdHEB70Sz+Y7vx+GG/XBkjublIlVTKi5Xd4IN4iduzBoHcgH+Cw8SB
# xClRIzgImLlTCLFm1BMIOQ8YnhV3
# SIG # End signature block
