﻿##### Scriptinfo ##################################################################################
# Scriptreihe	    PAM-AdminGUI\Setup-PAM&JEA
# Datum:            2017-11-15
# Version:          V1.04
# Programmierer:    Stephan Walther
###################################################################################################

# Variablen   
    notepad.exe PAM-AdminGUI.ini
    pause
 
    Get-Content -Path "PAM-AdminGUI.ini" | 
        ForEach-Object {
            if ($_.length -gt 0) {
                if ('[' -notcontains ($_)[0]) {
                    $Name,$Wert = ($_ -split '#')[0].Trim() -split '=='

                    $Name = "WS_" + $Name.trim()

                    Get-Variable -Scope global | Where-Object {$_.name -eq $Name} | Remove-Variable -Scope global
                    New-Variable -Name $Name.trim() -Value $Wert.trim() -Scope global
                }
            }
        }
        
# Setup Privileged Access Management - PAM
    # Vorbereitung PAM  
        Import-Module -Name ActiveDirectory 
        $ADFunktionsEbene = (Get-ADForest).ForestMode
        if ($ADFunktionsEbene -replace '\D' -lt 2016) {
            $Antwort = Read-Host -Prompt "Die Funktionsebene des AD ist $ADFunktionsEbene, es wird aber mind. Windows2016Forest benoetigt. Soll die Funktionsebene heraufgestuft werden? Der Vorgang ist nicht umkehrbar!!! (j|n)"
            if ($Antwort -eq 'j') {
                Set-ADForestMode -ForestMode "Windows2016Forest" -Identity (Get-ADForest).name
            } else {
                exit
            }         
        }
        if ((Get-ADOptionalFeature -Identity 'Privileged Access Management Feature').EnabledScopes.count -eq 0) {
            $Antwort = Read-Host -Prompt "Das Privileged Access Management Feature ist nicht aktiviert. Soll ich es aktivieren? Der Vorgang ist nicht umkehrbar!!! (j|n)"
            if ($Antwort -eq 'j') {
                Enable-ADOptionalFeature -Identity 'Privileged Access Management Feature' -Target (Get-ADForest).name -Scope ForestOrConfigurationSet
                Wait-Event -Timeout 5
            } else {
                exit   
            }
        }

        if ((Get-ADOptionalFeature -Identity 'Privileged Access Management Feature').EnabledScopes.count -eq 0) {
            Write-Host "PAM ist nicht aktiviert!" -ForegroundColor Yellow
            exit
        }

# Setup Just Enough Administration - JEA
    # erstelle die AdminRollenGruppe
        New-ADGroup `
            -Name "$WS_PAMAdminRolleGruppeName" `
            -GroupCategory Security `
            -GroupScope DomainLocal `
            -Path $WS_PAMAdminRolleGruppeOU -Description "Recht für JEA/PAM-AdminGUI"

    # erstelle PS-Module für die neue Rolle
        New-Item           -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI" -ItemType Directory |Out-Null
        New-ModuleManifest -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI.psd1"
        New-Item           -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\RoleCapabilities" -ItemType Directory | Out-Null

    # Definition der Administrations-Rolle
        $Rolle = @{
            Author = $env:username
            CompanyName = (Get-ADDomain).NetBIOSName
            ModulesToImport = 'ActiveDirectory'
            VisibleCmdlets = 'get-ADGroupMember','add-member'
            ScriptsToProcess = "C:\Program Files\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI-Functions.ps1"
            FunctionDefinitions = @{ Name = 'Get-UserInfo'; ScriptBlock = { $PSSenderInfo } }
        }
    # erzeuge die Rollenbeschreibungsdatei
        New-PSRoleCapabilityFile -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\RoleCapabilities\PAM-AdminGUI.psrc" @Rolle

    # kopiere FunctionsFile
        Copy-Item PAM-AdminGUI-Functions.ps1 -Destination "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI-Functions.ps1"
        Copy-Item PAM-AdminGUI.ini           -Destination "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI.ini"

    # erstelle Endpunkt-Sessionconfiguration
        $JEAConfigParameter = @{
            SessionType = 'RestrictedRemoteServer'
            RunAsVirtualAccount = $true
            RoleDefinitions = @{ "$((Get-ADDomain).NetBIOSName)\$WS_PAMAdminRolleGruppeName" = @{ RoleCapabilities = "PAM-AdminGUI"  } }
            TranscriptDirectory = "$env:ProgramData\JEAConfiguration\Transcripts"
        }
    
    # erstelle Verzeichnis für die Session-Konfigurationen
        New-Item -Path "$env:ProgramData\JEAConfiguration" -ItemType Directory -ErrorAction SilentlyContinue | Out-Null

    # erstelle JEA-Endpunkt
        if (Get-PSSessionConfiguration -Name 'PAM-AdminGUI' -ErrorAction SilentlyContinue) {
            Unregister-PSSessionConfiguration -Name 'PAM-AdminGUI' -ErrorAction Stop
        }
        New-PSSessionConfigurationFile -Path "$env:ProgramData\JEAConfiguration\PAM-AdminGUI.pssc" @JEAConfigParameter

    # lade die JEA-Session-Konfiguration
        Register-PSSessionConfiguration -Name 'PAM-AdminGUI' -Path "$env:ProgramData\JEAConfiguration\PAM-AdminGUI.pssc" -WarningAction SilentlyContinue

    # aktiviere ModuleLogging (optional)
        New-Item -Path HKLM:\SOFTWARE\Policies\Microsoft\Windows\Powershell -ErrorAction SilentlyContinue | out-null
        New-Item -Path HKLM:\SOFTWARE\Policies\Microsoft\Windows\Powershell\ModuleLogging -ErrorAction SilentlyContinue | out-null
        New-Item -Path HKLM:\SOFTWARE\Policies\Microsoft\Windows\Powershell\ModuleLogging\ModuleNames -ErrorAction SilentlyContinue | out-null
        Set-ItemProperty -Path HKLM:\SOFTWARE\Policies\Microsoft\Windows\Powershell\ModuleLogging\ModuleNames -Name '*' -Value '*' | Out-Null

# SIG # Begin signature block
# MIII4QYJKoZIhvcNAQcCoIII0jCCCM4CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU/ItKdYahz60+x/lZEEQGuBMm
# o06gggY/MIIGOzCCBSOgAwIBAgITGwAAAOBAvnzebcAjSAABAAAA4DANBgkqhkiG
# 9w0BAQsFADBUMRMwEQYKCZImiZPyLGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYC
# d3MxKTAnBgNVBAMTIFdTLUlUUy1aZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExMB4X
# DTE3MDUxMDA1NDI0OVoXDTE5MDUxMDA1NTI0OVowdTETMBEGCgmSJomT8ixkARkW
# A2l0czESMBAGCgmSJomT8ixkARkWAndzMQswCQYDVQQLEwJXUzERMA8GA1UECxMI
# QmVudXR6ZXIxDzANBgNVBAsTBldTLUlUUzEZMBcGA1UEAxMQV2FsdGhlciwgU3Rl
# cGhhbjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKqnKBq0nJLvqGwO
# aJkLU2174/KhLd4ykJZ2BgG4wse7s3ALM7dH1vxNmeKIoB09yk9nYZzDeRjZ7w7a
# WKgko8zvNHJCleQNtG6AReOoMIRuYyrbUmbKlmICn9V/ENUNQ6s1tiqnAIChb0J2
# 05GMaBYG+IFS+uLDzlTIsoaEKZjV3htgBEVQt6BYIdbsZ913ZUcpGWhQsCXKairT
# wB8pqPTaBCA5HJHyfAwxa+w2ERhF5q0r/xp9dEkemFjy0Xo9aQWUw3WPy8l3pTyp
# I2w/yDFqLfZu5z1QjWrcNA9VirSiM9PkOPXQb9riChc4PfFd90+JjeVhOlj3y0qb
# InQsfUMCAwEAAaOCAuMwggLfMA4GA1UdDwEB/wQEAwIHgDA8BgkrBgEEAYI3FQcE
# LzAtBiUrBgEEAYI3FQiIoH6B98FWhLmPAobjm2SE8KUnMYPz1QqE8ow3AgFkAgEC
# MB0GA1UdDgQWBBT1lyML2Qw04hBbAlX5PeqiA5IBQjAfBgNVHSMEGDAWgBS1Nrmv
# a6rveo5pQqKi2+u+kQ59cTCCASAGA1UdHwSCARcwggETMIIBD6CCAQugggEHhoHE
# bGRhcDovLy9DTj1XUy1JVFMtWmVydGlmaXppZXJ1bmdzc3RlbGxlLUNBMSgxKSxD
# Tj1XUy1DQTEsQ049Q0RQLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNl
# cnZpY2VzLENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NlcnRpZmljYXRl
# UmV2b2NhdGlvbkxpc3Q/YmFzZT9vYmplY3RDbGFzcz1jUkxEaXN0cmlidXRpb25Q
# b2ludIY+aHR0cDovL2NybC53cy5pdHMvY3JsZC9XUy1JVFMtWmVydGlmaXppZXJ1
# bmdzc3RlbGxlLUNBMSgxKS5jcmwwgc0GCCsGAQUFBwEBBIHAMIG9MIG6BggrBgEF
# BQcwAoaBrWxkYXA6Ly8vQ049V1MtSVRTLVplcnRpZml6aWVydW5nc3N0ZWxsZS1D
# QTEsQ049QUlBLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNlcnZpY2Vz
# LENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NBQ2VydGlmaWNhdGU/YmFz
# ZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9uQXV0aG9yaXR5MBMGA1UdJQQMMAoG
# CCsGAQUFBwMDMBsGCSsGAQQBgjcVCgQOMAwwCgYIKwYBBQUHAwMwKQYDVR0RBCIw
# IKAeBgorBgEEAYI3FAIDoBAMDnN0ZXBoYW5Ad3MuaXRzMA0GCSqGSIb3DQEBCwUA
# A4IBAQBjyTnnRXeyMEPW8SziBVFJcsX0K3XLCXz7PEl6sVuiWTHY3n9IOwJJ2sqF
# 3Sea0bpWrDFnJQtB+S4FXW6A/bVvuPlGMBtZ7T9DowowAQAmWvC/OJeeHGz2xyjh
# fDYAmMi3DXrVIEv+yWHsrh6CSt0KiHjMOh/Kk092AmyyPSRZpqmabzcoY7fI6xff
# ckOvS+Q3233ytXfS2YMuNKL8EuGaUhGj8Smb3grUtNW0kyJmL2wTgyWdUerbGpC3
# Ss7iv53hJXSoe8Rmo8spntb/5e+zt6q0ThqI4TN6sm9Gqgg8QcfzGc0XvW7V2GKU
# LT75dOMzV86lFZo+rvq1SaZb2GRcMYICDDCCAggCAQEwazBUMRMwEQYKCZImiZPy
# LGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYCd3MxKTAnBgNVBAMTIFdTLUlUUy1a
# ZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExAhMbAAAA4EC+fN5twCNIAAEAAADgMAkG
# BSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJ
# AzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMG
# CSqGSIb3DQEJBDEWBBRzPg4VqmSpHukBnE4GHe47eRcrrzANBgkqhkiG9w0BAQEF
# AASCAQBm1hjGOon51KGKo38kfrJYwYiq23mmCfzkKGUwwWUMN586jK/5aZNqGIjO
# 6b/SfugtZFTyz/XzyF5vIvBCnQJ3vJJL//3lOlwweYf0TzetPKZoHj/Q6UJBke7B
# YvbovAXoyXByYS5K9havN/Up/GSIU3lD1FNbMpkiNJa6VztzEj54IAZOl2w4i8BA
# 4UuITfqyUjmqiIvIxhSB+XAHhIJCSEJxXCbT+hI4ItqV9FYnDeAtdnHuGVFbrxdN
# P/i8AC4+lKdVwn/oUV7mAEmgn5SMmUfolEuMDTtRoiBwc3X+nerM18890geARQRi
# SOHvAAYifMPTbtBKcMOeuvBYM49a
# SIG # End signature block
